openerp.print_engine = function (instance, local) {
    local.SocketManager = instance.web.Class.extend({
        init: function () {
            this.socket = null;
            this.start();
        },
        start: function () {
            this.open_socket();
        },
        open_socket: function () {
            var self = this;

            var set_socket = function (socket) {
                var protocol = window.location.protocol;
                self.socket = io(protocol + '//' + socket.host + ':' + socket.port, {
                    path: socket.path
                });
                self.socket.on('connect', self.handle_connect);
                self.socket.on('connect_error', self.handle_connect_error);
                self.socket.on('show-printers', self.handle_printers);
                self.socket.on('print-error', self.handle_print_error);
                self.socket.on('print-finished', self.handle_print_finished);
            }

            this.get_socket_config().then(set_socket);
        },
        handle_connect: function () {
            $('#printer-status').removeClass();
            $('#printer-status').addClass('printer-status-online');
        },
        handle_connect_error: function () {
            $('#printer-status').removeClass();
            $('#printer-status').addClass('printer-status-offline');
        },
        handle_printers: function (printers) {
            console.log(printers);
        },
        handle_print_error: function () {
            console.log('error');
        },
        handle_print_finished: function () {
            console.log('finished');
        },
        get_socket_config: function () {
            var url = '/print_engine/socket_config';
            return this.get_from_server(url);
        },
        get_from_server: function (url) {
            instance.web.blockUI();
            return $.Deferred(function (d) {
                instance.web.jsonRpc(url, 'call').done(function () {
                    instance.web.unblockUI();
                    d.resolve.apply(d, arguments);
                }).fail(function (e) {
                    instance.web.unblockUI();
                    d.reject.apply(d, arguments);
                });
            });
        },
        request_print: function (b64_data) {
            if (!this.socket || !this.socket.connected) {
                instance.web.notification.do_warn('Impresión', 'La impresora no está preparada');
                return;
            }

            try {
                this.socket.emit('request-print', {
                    data: b64_data
                });
            } catch (e) {
                instance.webclient.crashmanager.show_message(e);
            }
        }
    });

    local.PrinterTopWidget = instance.web.Widget.extend({
        template: 'printEngine.PrinterTopWidget',
        init: function (parent) {
            this._super(parent)
        },
        start: function () {
            // this.$el.click(this, this.selectDefaultPrinter);
        }
    });

    if (instance.web) {
        if (instance.web.UserMenu) {
            instance.web.UserMenu.include({
                do_update: function(){
                    var printer = new local.PrinterTopWidget(this);
                    printer.appendTo($('.oe_systray'));
                    local.socket = new local.SocketManager()
                    return this._super.apply(this, arguments);
                }
            });
        }

        if (instance.web.ActionManager) {
            instance.web.ActionManager.include({
                ir_actions_report_xml: function (action, options) {
                    var url = '/print_engine/get_pdf';
                    var ctx = _.clone(action.context);

                    ctx.report_name = action.report_name;

                    instance.web.jsonRpc(url, 'call', {
                        context: ctx
                    }).done(function (result) {
                        instance.print_engine.socket.request_print(result.data);
                    }).fail(function (e) {
                        instance.webclient.crashmanager.show_message(e);
                    });
                }
            });
        }
    }
}